

//============================================================================
//
// MetaStock MSX for the Enhanced Hilbert Cycle Period 
// and the Enhanced Hilbert Channel Breakout Signal High and Low Functions
//
//============================================================================

#include <windows.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include "Hilbert.h"

#include "MSXStruc.h"  // required MSX definitions

// we don't want C++ name-mangling
#ifdef __cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------
// DLLMain
//
// This function is called when MetaStock (or MSXTest) loads and unloads
//   this DLL.  Global variable initialization, memory allocation, or
//   other initialization can be placed here.
// The return value (l_bRtrn) is used only when the function is called
//   with DLL_PROCESS_ATTACH, otherwise it is ignored.  See your Windows
//   programming documentation for more information regarding DllMain.
//------------------------------------------------------------------------
	
BOOL WINAPI DllMain( HANDLE a_hModule, 
                     DWORD  a_fdwReason, 
                     LPVOID a_lpReserved
				   )
{
    BOOL   l_bRtrn = TRUE;

	switch (a_fdwReason)
	{
	case DLL_PROCESS_ATTACH:
		// Code placed here will execute when the DLL is first loaded by MetaStock (or MSXTest).
		// If your DLL requires initialization of global variables or other start-up
		//   processing, do that here.
		

		break;
	case DLL_PROCESS_DETACH:
		// Code placed here will execute when the DLL is unloaded by MetaStock (or MSXTest).
		// If you allocated memory in DLL_PROCESS_ATTACH, deallocate it here.
		break;
	default:
		break;
	}

	return l_bRtrn;
}

//------------------------------------------------------------------------
//
// MSXInfo
//
// Required MSX function.  
//
//------------------------------------------------------------------------

BOOL __stdcall MSXInfo (MSXDLLDef *a_psDLLDef)
{
	// Stores Equis copyright information...
	strncpy (a_psDLLDef->szCopyright, "Copyright (c) Equis International 2000",
			sizeof(a_psDLLDef->szCopyright)-1);

	// Sets the number of functions to export
	a_psDLLDef->iNFuncs = 3; // 3 functions will be exported.
	a_psDLLDef->iVersion = MSX_VERSION;
	return MSX_SUCCESS;
}

//------------------------------------------------------------------------
//
// MSXNthFunction
//
// Required MSX function.
//
// MSXNthFunction is called once for each function specified in the
//   MSXInfo function.  On each call, a_iNthFunc will indicate the
//   zero-based function to define.
// The value copied into a_psFuncDef->szFunctionName must EXACTLY match
//   the exported name of the function, including case.
//
//------------------------------------------------------------------------

BOOL __stdcall MSXNthFunction (int a_iNthFunc, MSXFuncDef *a_psFuncDef)
{
	int l_bRtrn = MSX_SUCCESS; // no error

	switch (a_iNthFunc)
	{
	// One case entry (starting at 0) exists for each exported function.
	case 0:
		strcpy (a_psFuncDef->szFunctionName, "EnhancedHilbertPeriod");
		strcpy (a_psFuncDef->szFunctionDescription, "Enhanced Hilbert Cycle Period Function");
		a_psFuncDef->iNArguments = 1;
		break;
	case 1:
		strcpy (a_psFuncDef->szFunctionName, "ChannelHigh");
		strcpy (a_psFuncDef->szFunctionDescription, "Enhanced Hilbert Channel Breakout High Function");
		a_psFuncDef->iNArguments = 3;
		break;
	case 2:
		strcpy (a_psFuncDef->szFunctionName, "ChannelLow");
		strcpy (a_psFuncDef->szFunctionDescription, "Enhanced Hilbert Channel Breakout Low Function");
		a_psFuncDef->iNArguments = 3;
		break;
	default:
		l_bRtrn = MSX_ERROR; 
		break;
	}
	return l_bRtrn;
}


//------------------------------------------------------------------------
//
// MSXNthArg
//
// Required function only if one or more exported functions require
//   arguments. Functions and arguments are numbered from 0.
//   
// 
//------------------------------------------------------------------------

BOOL __stdcall MSXNthArg (int a_iNthFunc, int a_iNthArg,
                         MSXFuncArgDef *a_psFuncArgDef)
{
	BOOL l_bRtrn = MSX_SUCCESS;

	a_psFuncArgDef->iNCustomStrings = 0; 

	// One case entry defined for each function that has arguments,
	// And case entries for each argument.
	
	switch (a_iNthFunc)
	{
   case 0: // EnhancedHilbertPeriod
     switch (a_iNthArg)
     {
     case 0: // EnhancedHilbertPeriod:Arg0
       a_psFuncArgDef->iArgType = MSXDataArray; // PriceDataArray;
       strcpy (a_psFuncArgDef->szArgName, "PriceDataArray");
       break;
     default:
       l_bRtrn = MSX_ERROR; 
       break;
     }
     break;
   case 1: // ChannelHigh
     switch (a_iNthArg)
     {
     case 0: // ChannelHigh:Arg0
       a_psFuncArgDef->iArgType = MSXDataArray; // UserPriceArray
       strcpy (a_psFuncArgDef->szArgName, "UserPriceArray");
       break;
     case 1: // ChannelHigh:Arg1
       a_psFuncArgDef->iArgType = MSXNumeric; // Multiplier
       strcpy (a_psFuncArgDef->szArgName, "PeriodMultiplier");
       break;
     case 2: // ChannelLow:Arg2
       a_psFuncArgDef->iArgType = MSXNumeric; // AlternatePeriod
       strcpy (a_psFuncArgDef->szArgName, "AlternatePeriod");
       break;
     default:
       l_bRtrn = MSX_ERROR; 
       break;
     }
     break;
   case 2: // ChannelLow
     switch (a_iNthArg)
     {
     case 0: // ChannelLow:Arg0
       a_psFuncArgDef->iArgType = MSXDataArray; // UserPriceArray
       strcpy (a_psFuncArgDef->szArgName, "UserPriceArray");
       break;
     case 1: // ChannelLow:Arg1
       a_psFuncArgDef->iArgType = MSXNumeric; // Multiplier
       strcpy (a_psFuncArgDef->szArgName, "PeriodMultiplier");
       break;
     case 2: // ChannelLow:Arg2
       a_psFuncArgDef->iArgType = MSXNumeric; // AlternatePeriod
       strcpy (a_psFuncArgDef->szArgName, "AlternatePeriod");
       break;
     default:
       l_bRtrn = MSX_ERROR; 
       break;
     }
     break;

	default:
		l_bRtrn = MSX_ERROR; 
		break;
	}
	
	return l_bRtrn;
}


// ************************************************************************
// This local utility function is used to help ensure that no overflows
//   or underflows will occur during calcuations.  The MSXTest program
//   Stress Test function will call your DLL with a wide range of values,
//   including positive and negative values of FLT_MAX and FLT_MIN.
// Perform all intermediate calculations using doubles and then force the
//   results into the range of a float.
// Locally defined macros are used to avoid compiler definition differences.
// ************************************************************************
#define MSXMax(a,b) (((a) > (b)) ? (a) : (b))
#define MSXMin(a,b) (((a) < (b)) ? (a) : (b))
double ForceFloatRange (double a_lfDbl)
{
	if (a_lfDbl > 0.0)
	{
		a_lfDbl = MSXMin (a_lfDbl, double(FLT_MAX)); // make sure pos number <= FLT_MAX
		a_lfDbl = MSXMax (a_lfDbl, double(FLT_MIN)); // make sure pos number >= FLT_MIN
	}
	else
	{
		if (a_lfDbl < 0.0)
		{
			a_lfDbl = MSXMax (a_lfDbl, double(-FLT_MAX)); // make sure neg number >= -FLT_MAX
			a_lfDbl = MSXMin (a_lfDbl, double(-FLT_MIN)); // make sure neg number <= -FLT_MIN
		}
	}

	return a_lfDbl;
}


/////////////////////////////////////////////////////////////////////////
// 
// Function: EnhancedHilbertPeriod
//
// Description:
// The Enhanced Hilbert Cycle Period Function used by the Enhanced Hilbert
// Channel Breakout system
//
// Arguments:
// PriceDataArray - Array of price data (high, low, close, etc.) 
//                  to calculate cycle period from
//
/////////////////////////////////////////////////////////////////////////

BOOL __stdcall EnhancedHilbertPeriod (const MSXDataRec *a_psDataRec,
                              const MSXDataInfoRecArgsArray *a_psDataInfoArgs,
                              const MSXNumericArgsArray *a_psNumericArgs,
                              const MSXStringArgsArray *a_psStringArgs,
                              const MSXCustomArgsArray *a_psCustomArgs,
                              MSXResultRec *a_psResultRec)
{
	BOOL l_bRtrn = MSX_SUCCESS;

	// Variables to hold a_psDataInfoArgs[0]
	float *l_pfPrice = NULL;
	float *l_pfPeriod = NULL;
	int    l_iFirstValid = 0;
	int    l_iLastValid = 0;
	BOOL   l_bResult;
	int    l_iArray_Size;

	// if the right number of args are passed in and the array is not empty
	if ( a_psDataInfoArgs->iNRecs == 1 &&
		 a_psNumericArgs->iNRecs == 0 &&
		 a_psStringArgs->iNRecs == 0 &&
		 a_psCustomArgs->iNRecs == 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid >= 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid > a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid + 6 )
	{
		// then assign the array to local values
		l_pfPrice = a_psDataInfoArgs->psDataInfoRecs[0]->pfValue;
		l_iFirstValid = a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid;
		l_iLastValid = a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid;
		l_iArray_Size = l_iLastValid + 1;
		l_pfPeriod = new float [l_iArray_Size];
		ZeroMemory( l_pfPeriod, sizeof(float) * l_iArray_Size);

		//Call the Period Calculation Function
		l_bResult = CalculateHilbertPeriod(l_pfPrice, l_iFirstValid, l_iLastValid, l_pfPeriod, l_iArray_Size);


		// Store results in the a_psResultRec structure
		memcpy (a_psResultRec->psResultArray->pfValue, l_pfPeriod, sizeof (float) * (l_iLastValid + 1 ));
		a_psResultRec->psResultArray->iFirstValid = l_iFirstValid + 6;
		a_psResultRec->psResultArray->iLastValid = l_iLastValid;

		delete l_pfPeriod;

	}
	else // We've recieved invalid arguments
	{
		// Set error message
		strncpy (a_psResultRec->szExtendedError, 
                 "Invalid Argument(s)",
                 sizeof(a_psResultRec->szExtendedError)-1);

		// Invalidate the result array, and set the return code to MSX_ERROR
		a_psResultRec->psResultArray->iFirstValid = 0;
		a_psResultRec->psResultArray->iLastValid = -1;
		l_bRtrn = MSX_ERROR;
	}
	
	if (l_bResult != MSX_SUCCESS) // CalculateHilbertPeriod Function Failed
	{
	// Set error message
	strncpy (a_psResultRec->szExtendedError, 
             "Hilbert Period Calculation Failed",
             sizeof(a_psResultRec->szExtendedError)-1);

	// Invalidate the result array, and set the return code to MSX_ERROR
	a_psResultRec->psResultArray->iFirstValid = 0;
	a_psResultRec->psResultArray->iLastValid = -1;
	l_bRtrn = MSX_ERROR;
	}

	return l_bRtrn;

}

/////////////////////////////////////////////////////////////////////////
// 
// Function: CalculateHilbertPeriod
//
// Description:
// Performs the actual calculation of the Hilbert Cycle Period
// called from the Period, High or Low functions
//
// Arguments:
// const float *a_pfPrice - Array of Price data to use in calculation
// const int    a_iFirstValid - First valid element in a_pfPrice
// const int    a_iLastValid - Last valid element in a_pfPrice
// float       *a_pfPeriod - Array of Period data to be populated
// const int    a_iArraySize - Actual size of a_pfPeriod
//
/////////////////////////////////////////////////////////////////////////

BOOL CalculateHilbertPeriod(const float *a_pfPrice,
						    const int    a_iFirstValid,
						    const int    a_iLastValid,
						    float       *a_pfPeriod,
						    const int    a_iArraySize)

{

	BOOL l_bRtrn = MSX_SUCCESS;

		// Declare local calculation variables
		int       l_iCurrentBar = 0;
		int       l_iPreviousBar = 0;  // Stores l_iCurrentBar - 1
		double    *l_pdSmoother = NULL;
		double    *l_pdDetrender = NULL;
		double	  *l_pd_I1 = NULL;
		double	  *l_pd_Q1 = NULL;
		double	  *l_pd_jI = NULL;
		double	  *l_pd_jQ = NULL;
		double	  *l_pd_I2 = NULL;
		double	  *l_pd_Q2 = NULL;
		double	  *l_pd_X1 = NULL;
		double	  *l_pd_X2 = NULL;
		double	  *l_pd_Y1 = NULL;
		double	  *l_pd_Y2 = NULL;
		double	  *l_pd_Re = NULL;
		double	  *l_pd_Im = NULL;
		
		#define	  PI 3.14159265359f

		//  Allocate Memory for each array
		l_pdSmoother = new double [a_iArraySize];
		l_pdDetrender = new double [a_iArraySize];
		l_pd_I1 = new double [a_iArraySize];
		l_pd_Q1 = new double [a_iArraySize];
		l_pd_jI = new double [a_iArraySize];
		l_pd_jQ = new double [a_iArraySize];
		l_pd_I2 = new double [a_iArraySize];
		l_pd_Q2 = new double [a_iArraySize];
		l_pd_X1 = new double [a_iArraySize];
		l_pd_X2 = new double [a_iArraySize];
		l_pd_Y1 = new double [a_iArraySize];
		l_pd_Y2 = new double [a_iArraySize];
		l_pd_Re = new double [a_iArraySize];
		l_pd_Im = new double [a_iArraySize];

		// Initialize each array
		ZeroMemory( l_pdSmoother, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pdDetrender, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_I1, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Q1, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_jI, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_jQ, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_I2, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Q2, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_X1, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_X2, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Y1, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Y2, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Re, sizeof(double) * a_iArraySize);
		ZeroMemory( l_pd_Im, sizeof(double) * a_iArraySize);
		
		// Main Calculation loop (begins after the fifth valid bar)
		for( l_iCurrentBar = a_iFirstValid + 5; l_iCurrentBar <= a_iLastValid; l_iCurrentBar++ )
		{
			// Set Previous Bar counter
			l_iPreviousBar = l_iCurrentBar - 1;

			// Calculate Smoother and Detrender values
			l_pdSmoother[l_iCurrentBar] = (4 * a_pfPrice[l_iCurrentBar] + 3 * a_pfPrice[l_iPreviousBar] + 
										  2 * a_pfPrice[l_iCurrentBar - 2] + a_pfPrice[l_iCurrentBar - 3])/10;
			l_pdDetrender[l_iCurrentBar] = (.25 * l_pdSmoother[l_iCurrentBar] + .75 * l_pdSmoother[l_iCurrentBar - 2] - 
										   .75 * l_pdSmoother[l_iCurrentBar - 4] - .25 * l_pdSmoother[l_iCurrentBar - 6])*
										   double(.046f * a_pfPeriod[l_iPreviousBar] + .332f);

			// Calculate InPhase and Quadrature components
			l_pd_Q1[l_iCurrentBar] = (.25 * l_pdDetrender[l_iCurrentBar] + .75 * l_pdDetrender[l_iCurrentBar - 2] - 
								      .75 * l_pdDetrender[l_iCurrentBar - 4] - .25 * l_pdDetrender[l_iCurrentBar - 6]) *
								     double(.046f * a_pfPeriod[l_iPreviousBar] + .332f);
			l_pd_I1[l_iCurrentBar] = l_pdDetrender[l_iCurrentBar - 3];

			// Advance the phase of l_pd_I1 and l_pd_Q1 by 90 degrees
			l_pd_jI[l_iCurrentBar] = .25 * l_pd_I1[l_iCurrentBar] + .75 * l_pd_I1[l_iCurrentBar - 2] - 
								     .75 * l_pd_I1[l_iCurrentBar - 4] - .25 * l_pd_I1[l_iCurrentBar - 6];
			l_pd_jQ[l_iCurrentBar] = .25 * l_pd_Q1[l_iCurrentBar] + .75 * l_pd_Q1[l_iCurrentBar - 2] - 
								     .75 * l_pd_Q1[l_iCurrentBar - 4] -.25 * l_pd_Q1[l_iCurrentBar - 6];

			// Phasor addition to equalize amplitude due to quadrature 
			// calculations (and 3 bar averaging)
			l_pd_I2[l_iCurrentBar] = l_pd_I1[l_iCurrentBar] - l_pd_jQ[l_iCurrentBar];
			l_pd_Q2[l_iCurrentBar] = l_pd_Q1[l_iCurrentBar] + l_pd_jI[l_iCurrentBar];

			// Smooth the I and Q components before applying the discriminator
			l_pd_I2[l_iCurrentBar] = ForceFloatRange (.15 * l_pd_I2[l_iCurrentBar] + .85 * l_pd_I2[l_iPreviousBar]);
			l_pd_Q2[l_iCurrentBar] = ForceFloatRange (.15 * l_pd_Q2[l_iCurrentBar] + .85 * l_pd_Q2[l_iPreviousBar]);

			// Homodyne Discriminator
			// Complex Conjugate Multiply
			l_pd_X1[l_iCurrentBar] = l_pd_I2[l_iCurrentBar] * l_pd_I2[l_iPreviousBar];
			l_pd_X2[l_iCurrentBar] = l_pd_I2[l_iCurrentBar] * l_pd_Q2[l_iPreviousBar];
			l_pd_Y1[l_iCurrentBar] = l_pd_Q2[l_iCurrentBar] * l_pd_Q2[l_iPreviousBar];
			l_pd_Y2[l_iCurrentBar] = l_pd_Q2[l_iCurrentBar] * l_pd_I2[l_iPreviousBar];
			l_pd_Re[l_iCurrentBar] = l_pd_X1[l_iCurrentBar] + l_pd_Y1[l_iCurrentBar];
			l_pd_Im[l_iCurrentBar] = l_pd_X2[l_iCurrentBar] - l_pd_Y2[l_iCurrentBar];

			// Smooth to remove any undesired cross products
			l_pd_Re[l_iCurrentBar] = ForceFloatRange (.2 * l_pd_Re[l_iCurrentBar] + .8 * l_pd_Re[l_iPreviousBar]);
			l_pd_Im[l_iCurrentBar] = ForceFloatRange (.2 * l_pd_Im[l_iCurrentBar] + .8 * l_pd_Im[l_iPreviousBar]);
			
			// Compute Cycle Period
			if ( l_pd_Im[l_iCurrentBar] != 0 && l_pd_Re[l_iCurrentBar] != 0 )
				a_pfPeriod[l_iCurrentBar] = float (ForceFloatRange (360 / ( atan(l_pd_Im[l_iCurrentBar] / l_pd_Re[l_iCurrentBar] ) * 
											(180.0 / PI))));

			if ( a_pfPeriod[l_iCurrentBar] > 1.5f * a_pfPeriod[l_iPreviousBar] )
				a_pfPeriod[l_iCurrentBar] = 1.5f * a_pfPeriod[l_iPreviousBar];

			if ( a_pfPeriod[l_iCurrentBar] < .67f * a_pfPeriod[l_iPreviousBar] )
				a_pfPeriod[l_iCurrentBar] = .67f * a_pfPeriod[l_iPreviousBar];

			if ( a_pfPeriod[l_iCurrentBar] < 6 )
				a_pfPeriod[l_iCurrentBar] = 6;

			if ( a_pfPeriod[l_iCurrentBar] > 50 )
				a_pfPeriod[l_iCurrentBar] = 50;

			a_pfPeriod[l_iCurrentBar] = .2f * a_pfPeriod[l_iCurrentBar] + .8f * a_pfPeriod[l_iPreviousBar];
	 
		}

		// Free up memory used by local variables
		delete l_pdSmoother;
		delete l_pdDetrender;
		delete l_pd_I1;
		delete l_pd_Q1;
		delete l_pd_jI;
		delete l_pd_jQ;
		delete l_pd_I2;
		delete l_pd_Q2;
		delete l_pd_X1;
		delete l_pd_X2;
		delete l_pd_Y1;
		delete l_pd_Y2;
		delete l_pd_Re;
		delete l_pd_Im;

		return l_bRtrn;
}

/////////////////////////////////////////////////////////////////////////
// 
// Function: ChannelHigh
//
// Description:
// The Enhanced Hilbert Channel Breakout System High Function
//
// Arguments:
// UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
// EntryK - Multiplier for Enhanced Hilbert Cycle Period
// EntryVal - Optional User-Defined Lookback Period
// 
/////////////////////////////////////////////////////////////////////////

BOOL __stdcall ChannelHigh (const MSXDataRec *a_psDataRec,
                            const MSXDataInfoRecArgsArray *a_psDataInfoArgs,
                            const MSXNumericArgsArray *a_psNumericArgs,
                            const MSXStringArgsArray *a_psStringArgs,
                            const MSXCustomArgsArray *a_psCustomArgs,
                            MSXResultRec *a_psResultRec)
{
	BOOL l_bRtrn = MSX_SUCCESS;

	// Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
	float  *l_pfPeriod;
	float  *l_pfPrice;
	float  *l_pfHigh;
	int    l_iPeriodFirstValid;
	int    l_iPeriodLastValid;
	int    l_iPriceFirstValid;
	int    l_iPriceLastValid;
	int    l_iHighFirstValid;
	int    l_iHighLastValid;
	double l_dEntryKInitial;
	double l_dEntryValInitial;
	int	   l_iArray_Size;
	BOOL   l_bResult;

	
	// if the right number of args are passed in and the Price and High arrays are not empty
	if ( a_psDataInfoArgs->iNRecs == 1 &&
		 a_psNumericArgs->iNRecs == 2 &&
		 a_psStringArgs->iNRecs == 0 &&
		 a_psCustomArgs->iNRecs == 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid >= 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid > a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid + 6 &&
		 a_psDataRec->sHigh.iFirstValid >= 0 &&
		 a_psDataRec->sHigh.iLastValid > 0)
	{
		// Assign a local pointer to the User-Defined Price data arg
		l_pfPrice = a_psDataInfoArgs->psDataInfoRecs[0]->pfValue;
		l_iPriceFirstValid = a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid;
		l_iPriceLastValid = a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid;

		l_iPeriodFirstValid = a_psDataRec->sHigh.iFirstValid;
		l_iPeriodLastValid = a_psDataRec->sHigh.iLastValid;

		l_iArray_Size = l_iPriceLastValid + 1;
		l_pfPeriod = new float [l_iArray_Size];
		ZeroMemory( l_pfPeriod, sizeof(float) * l_iArray_Size);
		
		//Calculate the Hilbert Cycle Period from the User-Defined Prices
		l_bResult = CalculateHilbertPeriod( l_pfPrice, 
											l_iPriceFirstValid, 
											l_iPriceLastValid, 
											l_pfPeriod, 
											l_iArray_Size);

		// Assign the High price data arg to the local array
		l_pfHigh = a_psDataRec->sHigh.pfValue;
		l_iHighFirstValid = a_psDataRec->sHigh.iFirstValid;
		l_iHighLastValid = a_psDataRec->sHigh.iLastValid;

		// Assign Numeric Args to local values
		l_dEntryKInitial = double(a_psNumericArgs->fNumerics[0]);
		l_dEntryValInitial = double(a_psNumericArgs->fNumerics[1]);
		
		// If either manipulator is out of range, it is given a default value
		if (l_dEntryKInitial > 100)
			l_dEntryKInitial = 100;
		if (l_dEntryValInitial > 100)
			l_dEntryValInitial = 100;
		if (l_dEntryKInitial < 0)
			l_dEntryKInitial = 0;
		if (l_dEntryValInitial < 0)
			l_dEntryValInitial = 0;

		// Local calculation variables
		int       l_iCurrentBar;
		int		  l_iCount;
		double	  l_dEntryVal;
		double    l_dEntryK;
		double    l_dEntryLookBack = 0;
		float     *l_pfEntryChannel = NULL;

		l_pfEntryChannel = new float [l_iPeriodLastValid + 1];

		// Main Calculation Loop
		for( l_iCurrentBar = l_iPeriodFirstValid; l_iCurrentBar <= l_iPeriodLastValid; l_iCurrentBar++ )
		{
			// Reinitialize variables for next iteration
			l_dEntryVal = l_dEntryValInitial;
			l_dEntryK = l_dEntryKInitial;
			l_dEntryLookBack = 0;

			// Calculate the Entry Look Back Period
			if ( l_dEntryVal != 0 )
				l_dEntryLookBack = l_dEntryVal;
			else
				l_dEntryLookBack = l_dEntryK * double(l_pfPeriod[l_iCurrentBar]);

			if ( l_dEntryLookBack < 1 )
				 l_dEntryLookBack = 1;
			
			// Set EntryChannel to the lowest possible value
			l_pfEntryChannel[l_iCurrentBar] = 0;

			// Find the highest value in the Entry Look Back Period
			for (l_iCount = l_iHighFirstValid; l_iCount <= int (l_dEntryLookBack); l_iCount++)
			{
				if ( l_iCurrentBar > l_iCount )
				{
					if ( l_pfEntryChannel[l_iCurrentBar] < l_pfHigh[l_iCurrentBar - l_iCount] )
						l_pfEntryChannel[l_iCurrentBar] = l_pfHigh[l_iCurrentBar - l_iCount];

				}
			}
		}

	// Give back the Entry Channel
	// Store results in the a_psResultRec structure
	memcpy (a_psResultRec->psResultArray->pfValue, l_pfEntryChannel, sizeof (float) * (l_iPeriodLastValid + 1 ));
	a_psResultRec->psResultArray->iFirstValid = l_iPeriodFirstValid +6;
	a_psResultRec->psResultArray->iLastValid = l_iPeriodLastValid;

	// Free up memory used by local variables
	delete l_pfEntryChannel;
	delete l_pfPeriod;

	}
	else // We've recieved invalid arguments
	{
		// Set error message
		strncpy (a_psResultRec->szExtendedError, 
                 "Invalid Argument(s)",
                 sizeof(a_psResultRec->szExtendedError)-1);

		// Invalidate the result array, and set the return code to MSX_ERROR
		a_psResultRec->psResultArray->iFirstValid = 0;
		a_psResultRec->psResultArray->iLastValid = -1;
		l_bRtrn = MSX_ERROR;
	}

	if (l_bResult != MSX_SUCCESS) // CalculateHilbertPeriod Function Failed
	{
	// Set error message
	strncpy (a_psResultRec->szExtendedError, 
             "Hilbert Period Calculation Failed",
             sizeof(a_psResultRec->szExtendedError)-1);

	// Invalidate the result array, and set the return code to MSX_ERROR
	a_psResultRec->psResultArray->iFirstValid = 0;
	a_psResultRec->psResultArray->iLastValid = -1;
	l_bRtrn = MSX_ERROR;
	}

	return l_bRtrn;
}


/////////////////////////////////////////////////////////////////////////
// 
// Function: ChannelLow
//
// Description:
// The Enhanced Hilbert Channel Breakout System Low Function
//
// Arguments:
// UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
// ExitK - Multiplier for Enhanced Hilbert Cycle Period
// ExitVal - Optional User-Defined Lookback Period
// 
/////////////////////////////////////////////////////////////////////////

BOOL __stdcall ChannelLow (const MSXDataRec *a_psDataRec,
                           const MSXDataInfoRecArgsArray *a_psDataInfoArgs,
                           const MSXNumericArgsArray *a_psNumericArgs,
                           const MSXStringArgsArray *a_psStringArgs,
                           const MSXCustomArgsArray *a_psCustomArgs,
                           MSXResultRec *a_psResultRec)
{
	BOOL l_bRtrn = MSX_SUCCESS;

	// Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
	float  *l_pfPeriod = NULL;
	float  *l_pfPrice;
	float  *l_pfLow;
	int    l_iPeriodFirstValid;
	int    l_iPeriodLastValid;
	int    l_iPriceFirstValid;
	int    l_iPriceLastValid;
	int    l_iLowFirstValid;
	int    l_iLowLastValid;
	int	   l_iArray_Size;
	double l_dExitKInitial;
	double l_dExitValInitial;
	BOOL   l_bResult;
	
	// if the right number of args are passed in and the Price and Low arrays are not empty
	if ( a_psDataInfoArgs->iNRecs == 1 &&
		 a_psNumericArgs->iNRecs == 2 &&
		 a_psStringArgs->iNRecs == 0 &&
		 a_psCustomArgs->iNRecs == 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid >= 0 &&
		 a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid > a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid + 6 &&
		 a_psDataRec->sLow.iFirstValid >= 0 &&
		 a_psDataRec->sLow.iLastValid > 0)
	{
		// Assign a local pointer to the User-Defined Price data arg
		l_pfPrice = a_psDataInfoArgs->psDataInfoRecs[0]->pfValue;
		l_iPriceFirstValid = a_psDataInfoArgs->psDataInfoRecs[0]->iFirstValid;
		l_iPriceLastValid = a_psDataInfoArgs->psDataInfoRecs[0]->iLastValid;

		l_iPeriodFirstValid = a_psDataRec->sLow.iFirstValid;
		l_iPeriodLastValid = a_psDataRec->sLow.iLastValid;

		l_iArray_Size = l_iPriceLastValid + 1;
		l_pfPeriod = new float [l_iArray_Size];
		ZeroMemory( l_pfPeriod, sizeof(float) * l_iArray_Size);

		//Calculate the Hilbert Cycle Period from the User-Defined Prices
		l_bResult = CalculateHilbertPeriod( l_pfPrice, 
											l_iPriceFirstValid, 
											l_iPriceLastValid, 
											l_pfPeriod, 
											l_iArray_Size);


		// Assign the Low price Arg to the local array
		l_pfLow = a_psDataRec->sLow.pfValue;
		l_iLowFirstValid = a_psDataRec->sLow.iFirstValid;
		l_iLowLastValid = a_psDataRec->sLow.iLastValid;

		// Assign Numeric Args to local values
		l_dExitKInitial = a_psNumericArgs->fNumerics[0];
		l_dExitValInitial = a_psNumericArgs->fNumerics[1];

		// If either manipulator is out of range, it is given a default value
		if (l_dExitKInitial > 100)
			l_dExitKInitial = 100;
		if (l_dExitValInitial > 100)
			l_dExitValInitial = 100;
		if (l_dExitKInitial < 0)
			l_dExitKInitial = 0;
		if (l_dExitValInitial < 0)
			l_dExitValInitial = 0;

		// Local calculation variables
		int        l_iCurrentBar;
		int		   l_iCount;
		double     l_dExitVal;
		double     l_dExitK;
		double     l_dExitLookBack = 0;
		float      *l_pfExitChannel = NULL;

		l_pfExitChannel = new float [l_iPeriodLastValid + 1];

		// Main Calculation Loop
		for( l_iCurrentBar = l_iPeriodFirstValid; l_iCurrentBar <= l_iPeriodLastValid; l_iCurrentBar++ )
		{
			// Reinitialize variables for next iteration
			l_dExitVal = l_dExitValInitial;
			l_dExitK = l_dExitKInitial;
			l_dExitLookBack = 0;

			// Calculate the Exit Look Back Period
			if ( l_dExitVal != 0 )
				l_dExitLookBack = l_dExitVal;
			else
				l_dExitLookBack = l_dExitK * double(l_pfPeriod[l_iCurrentBar]);

			if ( l_dExitLookBack < 1 )
				 l_dExitLookBack = 1;
			
			// Set ExitChannel to a sufficiently high value
			l_pfExitChannel[l_iCurrentBar] = 100000;

			// Find the lowest value in the Exit Look Back Period
			for (l_iCount = l_iLowFirstValid; l_iCount <= l_dExitLookBack; l_iCount++)
			{
				if ( l_iCurrentBar > l_iCount )
				{
					if ( l_pfExitChannel[l_iCurrentBar] > l_pfLow[l_iCurrentBar - l_iCount] )
						l_pfExitChannel[l_iCurrentBar] = l_pfLow[l_iCurrentBar - l_iCount];
				}
			}
		}

	// Give back the ExitChannel for use in the Expert Advisor
	// Store results in the a_psResultRec structure
	memcpy (a_psResultRec->psResultArray->pfValue, l_pfExitChannel, sizeof (float) * (l_iPeriodLastValid + 1 ));
	a_psResultRec->psResultArray->iFirstValid = l_iPeriodFirstValid +6;
	a_psResultRec->psResultArray->iLastValid = l_iPeriodLastValid;

	// Free up memory used by local variables
	delete l_pfExitChannel;
	delete l_pfPeriod;


	}

	else // We've recieved invalid arguments
	{
		// Set error message
		strncpy (a_psResultRec->szExtendedError, 
                 "Invalid Argument(s)",
                 sizeof(a_psResultRec->szExtendedError)-1);

		// Invalidate the result array, and set the return code to MSX_ERROR
		a_psResultRec->psResultArray->iFirstValid = 0;
		a_psResultRec->psResultArray->iLastValid = -1;
		l_bRtrn = MSX_ERROR;
	} 
	
	if (l_bResult != MSX_SUCCESS) // CalculateHilbertPeriod Function Failed
	{
	// Set error message
	strncpy (a_psResultRec->szExtendedError, 
             "Hilbert Period Calculation Failed",
             sizeof(a_psResultRec->szExtendedError)-1);

	// Invalidate the result array, and set the return code to MSX_ERROR
	a_psResultRec->psResultArray->iFirstValid = 0;
	a_psResultRec->psResultArray->iLastValid = -1;
	l_bRtrn = MSX_ERROR;
	}

	return l_bRtrn;
}

#ifdef __cplusplus
}
#endif

